package com.lantone.qc.kernel.structure.ai;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.lantone.qc.kernel.client.CRFServiceClient;
import com.lantone.qc.kernel.structure.ai.model.CrfOut;
import com.lantone.qc.kernel.structure.ai.model.EntityEnum;
import com.lantone.qc.kernel.structure.ai.process.*;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.doc.BeHospitalizedDoc;
import com.lantone.qc.pub.model.entity.*;
import com.lantone.qc.pub.model.label.*;
import com.lantone.qc.pub.model.vo.CRFVo;
import com.lantone.qc.pub.util.StringUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * @ClassName : InHospitalDoc
 * @Description :
 * @Author : 楼辉荣
 * @Date: 2020-03-03 18:58
 */
public class BeHospitalizedAI {
    /**
     * FirstCourseRecord_cx[病历首程]
     * PastFamily_cx[既往史家族史]
     * PersonalHistory_cx[个人史(月经史、婚育史)]
     * HPIForCX_cx[主诉、现病史、专科检查]
     * GeneralVital_cx[一般体格检查]
     * chief_present[邵逸夫医院主诉现病史]
     */
    public static List<String> medicalTextType = Arrays.asList("FirstCourseRecord_cx", "PastFamily_cx", "PersonalHistory_cx", "HPIForCX_cx",
            "GeneralVital_cx", "chief_present");
    public static String entityRelationObject = "entity_relation_object";
    public static String outputs = "outputs";

    public void medrec(InputInfo inputInfo,CRFServiceClient crfServiceClient){

        Map<String,List<CrfOut>> crfOut = new HashMap<>();//主诉-->
        JSONArray crfContent = new JSONArray();
        BeHospitalizedDoc beHospitalizedDoc = inputInfo.getBeHospitalizedDoc();
        String chief_text = beHospitalizedDoc.getChiefLabel().getText();
        String Personal_text = beHospitalizedDoc.getPersonalLabel().getText();
        String family_text = beHospitalizedDoc.getFamilyLabel().getText();
        String past_text = beHospitalizedDoc.getPastLabel().getText();
        String present_text = beHospitalizedDoc.getPresentLabel().getText();
        //月经史
        String menstrual_text = beHospitalizedDoc.getMenstrualLabel().getText();
        //婚育史
        String marital_text = beHospitalizedDoc.getMaritalLabel().getText();
        //一般体格检查
        String vital_text = beHospitalizedDoc.getVitalLabel().getText();
        //专科体格检查
        String vitalSpecial_text = beHospitalizedDoc.getVitalLabelSpecial().getText();
        //主诉
        putContent(crfContent,medicalTextType.get(3), chief_text,Content.chief);
        //现病史
        putContent(crfContent,medicalTextType.get(3), present_text,Content.present);
        //专科检查
        putContent(crfContent,medicalTextType.get(3), vitalSpecial_text,Content.special_exam);
        //既往史
        putContent(crfContent,medicalTextType.get(3), past_text,Content.past);
        //家族史
        putContent(crfContent,medicalTextType.get(3), family_text,Content.family);
        //存放一般查体
        putContent(crfContent,medicalTextType.get(3), vital_text,Content.phys_exam);
        //个人史
        putContent(crfContent,medicalTextType.get(3), Personal_text,Content.personal);
        //月经史
        putContent(crfContent,medicalTextType.get(3), menstrual_text,Content.menses);
        //婚育史
        putContent(crfContent,medicalTextType.get(3), marital_text,Content.marriage);
        //存储CRF完整所需结构数据
        CRFVo crfVo = new CRFVo();
        crfVo.setData(crfContent);
        //获取CRF模型返回数据
        JSONArray data = getAnnotation(crfServiceClient, crfVo).getData();
        JSONObject midData = getOutputs(data);
        //处理主诉
        putChiefCrfData(midData.getJSONObject(Content.chief), inputInfo);
        //处理现病史
        putPresentCrfData(midData.getJSONObject(Content.present), inputInfo);
        //处理既往史
        putPastCrfData(midData.getJSONObject(Content.past), inputInfo);
        //处理个人史
        putPersonCrfData(midData.getJSONObject(Content.personal), inputInfo);
        //处理家族史
        putFamilyCrfData(midData.getJSONObject(Content.family),inputInfo);



//        //存放CRF模型既往史、家族史返回数据
//        putAllCrfData(midData.getJSONObject(Content.past), crfOut);
//        putAllCrfData(midData.getJSONObject(Content.family), crfOut);
//        //存放CRF模型一般查体(体格检查（一）)返回数据
//        putAllCrfData(midData.getJSONObject(Content.phys_exam), crfOut);
//        //存放CRF模型个人史、月经史、婚育史返回数据
//        putAllCrfData(midData.getJSONObject(Content.personal), crfOut);
//        putAllCrfData(midData.getJSONObject(Content.menses), crfOut);
//        putAllCrfData(midData.getJSONObject(Content.marriage), crfOut);
//        //存放CRF模型病历特点、初步诊断、诊断依据返回数据
//        putAllCrfData(midData.getJSONObject(Content.case_feature), crfOut);
//        putAllCrfData(midData.getJSONObject(Content.pridiag), crfOut);
//        putAllCrfData(midData.getJSONObject(Content.diag_basis), crfOut);
//        return crfOut;
    }
    public void putChiefCrfData(JSONObject jsonObject, InputInfo inputInfo){
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        //放置入inputinfo
        ChiefLabel chiefLabel = inputInfo.getBeHospitalizedDoc().getChiefLabel();
        chiefLabel.setClinicals(loadClinicals(aiOut));
        chiefLabel.setDiags(loadDiags(aiOut));
    }

    public void putPresentCrfData(JSONObject jsonObject, InputInfo inputInfo){
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        //放置入inputinfo
        PresentLabel presentLabel = inputInfo.getBeHospitalizedDoc().getPresentLabel();
        presentLabel.setClinicals(loadClinicals(aiOut));
        presentLabel.setGenerals(loadGeneralDes(aiOut));
        presentLabel.setPacses(loadpacses(aiOut));
    }
    public void putFamilyCrfData(JSONObject jsonObject, InputInfo inputInfo){
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        //放置入inputinfo
        FamilyLabel familyLabel = inputInfo.getBeHospitalizedDoc().getFamilyLabel();
        familyLabel.setDiags(loadDiags(aiOut));
        familyLabel.setFamilies(loadFamily(aiOut));
        EntityProcess entityProcess = new EntityProcess();
        try {
            List<DiagInfectious> diagInfectious = entityProcess.addEntity(aiOut, EntityEnum.INFECTIOUS_KEYWORD, DiagInfectious.class);
            familyLabel.setDiagInfectious(diagInfectious);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public void putPastCrfData(JSONObject jsonObject, InputInfo inputInfo){
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        //放置入inputinfo
        EntityProcessPast entityProcessPast = new EntityProcessPast();
        inputInfo.getBeHospitalizedDoc().setPastLabel(entityProcessPast.extractEntity(aiOut));

    }
    public void putPersonCrfData(JSONObject jsonObject, InputInfo inputInfo){
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        //放置入inputinfo
        PersonalLabel personalLabel = inputInfo.getBeHospitalizedDoc().getPersonalLabel();
        personalLabel.setDiags(loadDiags(aiOut));

    }
    /**
     * 关系抽取临床表现信息
     * @param aiOut
     * @return
     */
    public List<Clinical> loadClinicals(JSONObject aiOut) {
        //主诉
        EntityProcessClinic entityProcessClinic = new EntityProcessClinic();
        List<Clinical> clinicals = entityProcessClinic.extractEntity(aiOut);
        return clinicals;
    }
    /**
     * 关系抽取临床表现信息
     * @param aiOut
     * @return
     */
    public List<Family> loadFamily(JSONObject aiOut) {

        EntityProcessFamily entityProcessFamily= new EntityProcessFamily();
        List<Family> families = entityProcessFamily.extractEntity(aiOut);
        return families;
    }
    /**
     * 关系抽取疾病信息
     * @param aiOut
     * @return
     */
    public List<Diag> loadDiags(JSONObject aiOut) {
        //诊断信息
        EntityProcessDiag entityProcessDiag = new EntityProcessDiag();
        List<Diag> diags = entityProcessDiag.extractEntity(aiOut);
        return diags;
    }
    /**
     * 关系抽取化验信息
     * @param aiOut
     * @return
     */
    public List<Lis> loadLises(JSONObject aiOut) {
        //化验信息
        EntityProcessLis entityProcessLis = new EntityProcessLis();
        List<Lis> lises = entityProcessLis.extractEntity(aiOut);
        return lises;
    }
    /**
     * 关系抽取辅检信息
     * @param aiOut
     * @return
     */
    public List<Pacs> loadpacses(JSONObject aiOut) {
        //辅检信息
        EntityProcessPacs entityProcessLis = new EntityProcessPacs();
        List<Pacs> pacses = entityProcessLis.extractEntity(aiOut);
        return pacses;
    }
    /**
     * 关系抽取体格检查
     * @param aiOut
     * @return
     */
    public List<Vital> loadvital(JSONObject aiOut) {
        //一般情况描述信息
        EntityProcessVital entityProcessVital = new EntityProcessVital();
        List<Vital> vitals = entityProcessVital.extractEntity(aiOut);
        return vitals;
    }
    /**
     * 关系抽取一般情况描述信息
     * @param aiOut
     * @return
     */
    public List<GeneralDesc> loadGeneralDes(JSONObject aiOut) {
        //一般情况描述信息
        EntityProcessGenerals entityProcessGeneral = new EntityProcessGenerals();
        List<GeneralDesc> generals = entityProcessGeneral.extractEntity(aiOut);
        return generals;
    }

    public void putContent(JSONArray crfContent, String medicalTextType, String text,String sign){
        String move_text = this.moveSpecialSymbols(text);
        if (StringUtil.isEmpty(move_text)) {
            return;
        }
        JSONObject detailContent = new JSONObject();
        detailContent.put("medical_text_type", medicalTextType);
        detailContent.put("content", move_text);
        detailContent.put("detail_title", sign);
        crfContent.add(detailContent);
    }
    private Annotation getAnnotation(CRFServiceClient crfServiceClient, CRFVo crfVo) {

        Annotation annotation = new Annotation();
        try {
            String annotation_str = crfServiceClient.getAnnotation(crfVo);
            annotation = JSON.parseObject(annotation_str, Annotation.class);
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return annotation;
        }
    }

    /**
     * 获取CRF模型输出并处理
     *
     * @param data
     * @return
     */
    public JSONObject getOutputs(JSONArray data) {
        JSONObject midData = new JSONObject();
        for (int i = 0; i < data.size(); i++) {
            JSONObject detailContent = data.getJSONObject(i);
            String detail_title = detailContent.getString("detail_title");
            if (StringUtil.isNotEmpty(detail_title) && midData.get(detail_title) == null) {
                midData.put(detail_title, detailContent);
            }
        }
        return midData;
    }
    //文本去除\r\n\t特殊符号
    public String moveSpecialSymbols(String content){
        if (StringUtil.isEmpty(content)){
            return "";
        }
        return content.replaceAll("\\r\\n|/r/n", "\r\n")
                .replaceAll("\\t", "\t").trim();
    }
}